// ************************************************************************
//
// misc_headers.h
//
// Part of Polyanna, Richard Gault, 2001
//
// Copyright (C) University of Oxford, 2001, 2002
//
// ************************************************************************

#ifndef MISC_HEADERS

#define MISC_HEADERS

#include <set>
#include <vector>
#include <iostream>

// ************************************************************************
// ***            Miscellaneous (non-member) functions                  ***
// ************************************************************************

// Just an abbreviation for natural numbers
typedef unsigned int nat;

// Return the highest common factor of two integers.  Uses Euclid's
// algorithm.
int highest_common_factor(int x, int y);

// Return the max of two integers
int max(int x, int y);

// Return the min of two integers
int min(int x, int y);

// The smallest element in the set s.  (Answer undefined if s is emoty.)
inline int min_set_value(const std::set<int>& s)
{
  return *(s.begin());
}

// The largest element in the set s.  (Answer undefined if s is empty.)
inline int max_set_value(const std::set<int>& s)
{
  // Stroustrup claims that "s.end()-1 " ought to work, but it doesn't.
  // (At least, not under gcc 2.95) This failure is the whole reason for
  // the existence of this function.
  std::set<int>::const_iterator si = s.end();
  return *(--si);
}

// Is set s a subset of set t?
bool is_subset(const std::set<int> &s, const std::set<int> &t);

// Output a tuple of ints on stream s
void display_tuple(std::ostream &s, const std::vector<int> &t);

// Output a set of ints on stream s
void display_set(std::ostream &s, const std::set<int> &c);

#endif

